//Douglas Fjällrud, Axel Blomén Grupp 35
package src.rsa;

import java.math.BigInteger;

public class RSAKeyGenerator {
    private BigInteger p, q, N, e, d;
    
    public RSAKeyGenerator(int bitLength) {
        generateKeys(bitLength);
    }

    private void generateKeys(int bitLength) {
        System.out.println("Genererar RSA-nycklar med " + bitLength + "-bitars primtal...");
    
        p = PrimeGenerator.generatePrime(bitLength);
        q = PrimeGenerator.generatePrime(bitLength);
        
        //Beräkna N = p * q
        N = p.multiply(q);
        
        //Beräkna Euler φ(N) = (p-1) * (q-1)
        BigInteger phi = (p.subtract(BigInteger.ONE)).multiply(q.subtract(BigInteger.ONE));
    
        //Välj publik exponent e (standard är 65537)
        e = new BigInteger("65537");
    
        //Beräkna privat exponent d = e⁻¹ mod φ(N) med utökade Euklides algoritm
        d = EuclideanAlgorithm.modularInverse(e, phi);
    
        System.out.println("Nycklar genererade!");
    }
    

    public BigInteger[] getPublicKey() {
        return new BigInteger[]{N, e};
    }
    
    public BigInteger[] getPrivateKey() {
        return new BigInteger[]{N, d};
    }
    
    public static void main(String[] args) {
        RSAKeyGenerator rsa = new RSAKeyGenerator(512);
    
        System.out.println("\n=== RSA Nycklar ===");
        System.out.println("Public Key:  N = " + rsa.getPublicKey()[0] + ", e = " + rsa.getPublicKey()[1]);
        System.out.println("Private Key: N = " + rsa.getPrivateKey()[0] + ", d = " + rsa.getPrivateKey()[1]);
    }
    
}
